<?php

namespace Drupal\records_tax\Entity;

use Drupal\records\Entity\StatefulContentEntityTrait;
use Drupal\records\MachineName\Field\Record as RecordField;
use Drupal\user\EntityOwnerTrait;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityChangedTrait;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\StringTranslation\TranslatableMarkup;

/**
 * The default implementation of the Tax Form content entity.
 *
 * phpcs:disable
 * @ContentEntityType(
 *   id = "records_tax_form",
 *   label = @Translation("Tax form"),
 *   label_collection = @Translation("Tax forms"),
 *   label_singular = @Translation("tax form"),
 *   label_plural = @Translation("tax forms"),
 *   label_count = @PluralTranslation(
 *     singular = "@count tax form",
 *     plural = "@count tax forms"
 *   ),
 *   bundle_label = @Translation("Tax form type", context = "Records"),
 *   bundle_entity_type = "records_tax_form_type",
 *   field_ui_base_route = "entity.records_tax_form_type.edit_form",
 *   handlers = {
 *     "list_builder" = "Drupal\records_tax\Entity\Controller\FormListBuilder",
 *     "views_data" = "Drupal\views\EntityViewsData",
 *     "access" = "Drupal\entity\EntityAccessControlHandler",
 *     "permission_provider" = "Drupal\entity\EntityPermissionProvider",
 *     "form" = {
 *       "default" = "Drupal\records_tax\Entity\Form\Form",
 *       "add" = "Drupal\records_tax\Entity\Form\Form",
 *       "edit" = "Drupal\records_tax\Entity\Form\Form",
 *       "delete" = "Drupal\Core\Entity\ContentEntityDeleteForm",
 *     },
 *     "route_provider" = {
 *       "default" = "Drupal\records_tax\Entity\Routing\FormRouteProvider",
 *     }
 *   },
 *   base_table = "records_tax_form",
 *   entity_keys = {
 *     "id" = "id",
 *     "bundle" = "type",
 *     "uuid" = "uuid",
 *     "uid" = "owner_id",
 *     "label" = "title",
 *     "owner" = "owner_id",
 *   },
 *   admin_permission = "administer records_tax_form",
 *   permission_granularity = "bundle",
 *   links = {
 *     "canonical" = "/tax-form/{records_tax_form}",
 *     "add-page" = "/tax-form/add",
 *     "add-form" = "/tax-form/add/{records_tax_form_type}",
 *     "edit-form" = "/tax-form/{records_tax_form}/edit",
 *     "delete-form" = "/tax-form/{records_tax_form}/delete",
 *     "collection" = "/admin/records/tax-forms",
 *   }
 * )
 *
 * @I Make the Tax Form entity revisionable
 *    type     : feature
 *    priority : high
 *    labels   : tax, tax-form
 * phpcs:enable
 */
class Form extends ContentEntityBase implements FormInterface {

  use EntityChangedTrait;
  use EntityOwnerTrait;
  use StatefulContentEntityTrait;

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(
    EntityTypeInterface $entity_type
  ) {
    $fields = parent::baseFieldDefinitions($entity_type);

    // We expose the type to the UI, view only.
    $fields[RecordField::TYPE]
      ->setLabel(new TranslatableMarkup('Type'))
      ->setDisplayConfigurable('view', TRUE);

    // Title/subject for the form. It may be programmatically generated by the
    // configurator plugin associated with the type of the record.
    $fields[RecordField::TITLE] = BaseFieldDefinition::create('string')
      ->setLabel(new TranslatableMarkup('Title'))
      ->setDescription(new TranslatableMarkup(
        'A short title explaining what the tax form is about.'
      ))
      ->setRequired(TRUE)
      ->setSetting('max_length', 255)
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields[RecordField::STATE] = BaseFieldDefinition::create('state')
      ->setLabel(new TranslatableMarkup('State'))
      ->setDescription(new TranslatableMarkup(
        'The state of the tax form.'
      ))
      ->setRequired(TRUE)
      ->setSetting(
        'workflow_callback',
        [__CLASS__, 'getWorkflowId']
      )
      ->setDisplayConfigurable('view', TRUE);

    // Add the Owner field.
    // The owner is essentially the assignee i.e. the user currently responsible
    // for the tax form. Information about who created and changed the form
    // would be kept once we make the entity revisionable.
    // phpcs:disable
    // @I Provide widget for only selecting users with appropriate roles
    // phpcs:enable
    $fields += static::ownerBaseFieldDefinitions($entity_type);
    $fields[RecordField::OWNER_ID]
      ->setLabel(new TranslatableMarkup('Owner'))
      ->setDescription(new TranslatableMarkup(
        'The user currently responsible for the tax form.'
      ))
      ->setRequired(TRUE)
      ->setDisplayConfigurable('view', TRUE)
      ->setDisplayOptions(
        'view',
        [
          'label' => 'hidden',
          'type' => 'author',
          'weight' => 0,
        ]
      )
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions(
        'form',
        [
          'type' => 'entity_reference_autocomplete',
          'weight' => 5,
          'settings' => [
            'placeholder' => 'Assign the tax form to a user',
          ],
        ]
      );

    // Timestamps.
    $fields[RecordField::CREATED] = BaseFieldDefinition::create('created')
      ->setLabel(new TranslatableMarkup('Created'))
      ->setDescription(new TranslatableMarkup(
        'The time that the tax form was created.'
      ))
      ->setDisplayConfigurable('view', TRUE);

    $fields[RecordField::CHANGED] = BaseFieldDefinition::create('changed')
      ->setLabel(new TranslatableMarkup('Changed'))
      ->setDescription(new TranslatableMarkup(
        'The time that the tax form was last edited.'
      ))
      ->setDisplayConfigurable('view', TRUE);

    return $fields;
  }

  /**
   * Returns the ID of the workflow for the state field.
   *
   * @param \Drupal\records_tax\Entity\FormInterface $form
   *   The tax form.
   *
   * @return string
   *   The workflow ID.
   */
  public static function getWorkflowId(FormInterface $form) {
    return \Drupal::service('entity_type.manager')
      ->getStorage('records_tax_form_type')
      ->loadWithPluginInstantiated($form->bundle())
      ->getPlugin()
      ->getPluginDefinition()['workflow_id'];
  }

}
