<?php

namespace Drupal\records\MachineName\Field;

/**
 * Holds machine names of fields reserved for any entity provided by Records.
 *
 * @link https://github.com/krystalcode/drupal8-coding-standards/blob/master/Fields.md#field-name-constants
 */
class Record {

  /**
   * Holds the machine name for the amount field.
   *
   * The meaning of the amount field might be different for different
   * entities. However, fields with this particular machine name can be
   * programmatically generated by the configurator plugin associated with the
   * entity's bundle configuration entity.
   */
  const AMOUNT = 'amount';

  /**
   * Holds the machine name for the created field.
   *
   * The timestamp that the record was first created.
   */
  const CREATED = 'created';

  /**
   * Holds the machine name for the changed field.
   *
   * The timestamp that the record was last updated.
   */
  const CHANGED = 'changed';

  /**
   * Holds the machine name for the currency field.
   *
   * Records e.g. income or expenses can be issued in a currency different than
   * that of the account. This field holds the currency of a record so that we
   * can convert it to the account currency when needed e.g. when generating
   * reports.
   */
  const CURRENCY = 'currency';

  /**
   * Holds the machine name for the currency exchange rate field.
   *
   * The exchange rate must be the one that will be used to convert FROM the
   * record currency TO the account currency. It should be set to NULL when
   * there is no conversion required i.e. the record currency is the same as the
   * account currency.
   */
  const CURRENCY_EXCHANGE_RATE = 'currency_exchange_rate';

  /**
   * Holds the machine name for the ID field.
   */
  const ID = 'id';

  /**
   * Holds the machine name for the title field.
   *
   * The meaning of the title field might be different for different
   * entities. However, fields with this particular machine name can be
   * programmatically generated by the configurator plugin associated with the
   * entity's bundle configuration entity.
   */
  const TITLE = 'title';

  /**
   * Holds the machine name for the tax field.
   *
   * Indicates whether the entity represents a tax. It only needs to be given a
   * value (TRUE) when the entity is tax. Otherwise, it can be left
   * empty. Queries collecting entities that are not tax should be querying for
   * entities with FALSE or NULL value on this field.
   */
  const IS_TAX = 'is_tax';

  /**
   * Holds the machine name for the taxable field.
   *
   * Indicates whether the entity is taxable. It only needs to be given a value
   * (TRUE) when the entity is taxable. Otherwise, it can be left empty. Queries
   * collecting entities that are not taxable should be querying for  entities
   * with FALSE or NULL value on this field.
   */
  const IS_TAXABLE = 'is_taxable';

  /**
   * Holds the machine name for the VAT-exempt field.
   *
   * Indicates whether the entity is exempt from VAT collection. It only needs
   * to be given a value (TRUE) when the entity is exempt. Otherwise, it can be
   * left empty. Queries collecting entities that are not exempt should be
   * querying for entities with FALSE or NULL value on this field.
   */
  const IS_VAT_EXEMPT = 'is_vat_exempt';

  /**
   * Holds the machine name for the items field.
   *
   * This field holds references to one or more items e.g. income items or
   * expense items. It is reserved for use as the main item-referencing field on
   * the parent record.
   */
  const ITEMS = 'items';

  /**
   * Holds the machine name for the normalized amount field.
   *
   * Records e.g. income or expenses can be issued in a currency different than
   * that of the account. This field holds the converted value that will be used
   * when we need the amount of a record in the primary currency of the account
   * e.g. when generating reports.
   *
   * This field should always be populated even when the record currency is the
   * same as the account currency. This makes generating reports easier i.e.
   * collecting values from one field only rather than having to collect values
   * from one field for some records and from another for others.
   *
   * @see \Drupal\records_currency\EntityConfigurator\Records\Plugin\PluginTrait\CurrencyTrait::setNormalizedAmount()
   */
  const NORMALIZED_AMOUNT = 'normalized_amount';

  /**
   * Holds the machine name for the owner ID field.
   *
   * The owner is essentially the assignee i.e. the user currently responsible
   * for the record. Information about who created and changed the record will
   * be kept once we make record entities revisionable.
   */
  const OWNER_ID = 'owner_id';

  /**
   * Holds the machine name for the reference field.
   *
   * The reference may be, for example, an invoice or receipt number.
   */
  const REFERENCE = 'reference';

  /**
   * Holds the machine name for the primary state field.
   */
  const STATE = 'state';

  /**
   * Holds the machine name for the type field (bundle).
   */
  const TYPE = 'type';

  /**
   * Holds the machine name for the UUID field.
   */
  const UUID = 'uuid';

}
